import time
from bs4 import BeautifulSoup
from typing import Union
from cloudscraper import create_scraper, Session
import re
from .utils import info_videotiktok
from requests.models import InvalidURL
class ssstik(Session):
    '''
    :param delay:
    ```python
    >>> tik=ssstik()
    >>> tik.get_media('....')
    [<[type:video]>, <[type:video]>, <[type:music]>]
    ```
    '''
    BASE = "https://ssstik.io"
    HEADERS = {"content-type": "application/x-www-form-urlencoded; charset=UTF-8","hx-active-element": "submit","hx-current-url": "https://ssstik.io/","hx-request": "true","hx-target": "target","origin": "https://ssstik.io","sec-fetch-dest": "","sec-fetch-mode": "cors","sec-fetch-site": "same-origin","user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36"}
    def __init__(self, delay: int=10, **kwargs) -> None:
        super().__init__(**kwargs)
        self.headers = self.HEADERS
        self.cf = create_scraper(delay=delay)
        self.html=self.cf.get(self.BASE)
        while True:
            if self.html.status_code == 403:
                print('retrying request')
                self.cf = create_scraper(delay=delay)
                self.html = self.html=self.cf.get(self.BASE)
                time.sleep(5)
            else:
                break
    def get_media(self, url: str)->list[info_videotiktok]:
        '''
        :param url:
        ```python
        >>> <ssstik object>.get_media('....')
        [<[type:video]>, <[type:video]>]
        ```
        '''
        try:
            post = self.cf.post(self.BASE+re.findall('hx-post=\"(.*?)\"',self.html.text)[0], data={"id": url,"locale": "en","tt": 0,"ts": 0})
            respon = BeautifulSoup(post.text, "html.parser")
            hasil  = [*[ info_videotiktok(url=i, Session=self.cf, type='video') for i in[ f'{self.BASE}{respon.find_all("a",class_="pure-button pure-button-primary is-center u-bl dl-button download_link without_watermark")[0].get("href")}',f'{respon.find_all("a",class_="pure-button pure-button-primary is-center u-bl dl-button download_link without_watermark_direct")[0].get("href")}']],info_videotiktok(f'{respon.find_all("a",class_="pure-button pure-button-primary is-center u-bl dl-button download_link music")[0].get("href")}', Session=self.cf, type='music')]
            return hasil 
        except IndexError:
            raise InvalidURL()




from bs4 import BeautifulSoup
from requests.sessions import Session
from tiktok_downloader.Except import InvalidUrl
from tiktok_downloader.utils import info_videotiktok
class keeptiktok:
    def __init__(self, url) -> None:
        self.request = Session()
        self.base_url = "https://keeptiktok.com/"
        self.headers = {"sec-ch-ua": '"Google Chrome";v="89", "Chromium";v="89", ";Not A Brand";v="99"',"sec-ch-ua-mobile": "?0","sec-fetch-dest": "document","sec-fetch-mode": "navigate","sec-fetch-site": "none","sec-fetch-user": "?1","upgrade-insecure-requests": "1","user-agent":"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36"}
        self.url = url
        self.token = BeautifulSoup(self.request.get(self.base_url, headers=self.headers).text, "html.parser").find_all("input", attrs={"id":"token"})[0]["value"]
    def get_info(self):
        try:
            headers = {"origin": "https://keeptiktok.com",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "referer": "https://keeptiktok.com/",
            "sec-ch-ua": '"Google Chrome";v="89", "Chromium";v="89", ";Not A Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "same-origin",
            "content-type": "application/x-www-form-urlencoded",
            "accept-language": "id-ID,id;q=0.9,en-US;q=0.8,en;q=0.7",
            "cache-control": "max-age=0",
            "sec-fetch-user": "?1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36"}
            return [info_videotiktok(BeautifulSoup(self.request.post("https://keeptiktok.com/index.php",headers=headers,  allow_redirects=True,data={"url":self.url, "token":self.token}).text, "html.parser").find_all("source")[0]["src"], self.headers, self.request)]
        except IndexError:
            raise InvalidUrl("URL ERROR")
    def __str__(self) -> str:
        return "<[ METHOD: KEEPTIKTOK] >"
    def __repr__(self) -> str:
        return self.__str__()


from requests import Session
import requests, json
from datetime import datetime
from bs4 import BeautifulSoup
import re
from tiktok_downloader.Except import InvalidUrl
from tiktok_downloader.keeptiktok import keeptiktok
from tiktok_downloader.utils import info_videotiktok
class info_post(requests.Session):
    def __init__(self, url: str) -> None:
        super().__init__()
        '''
        :param url: video url(tiktok)
        '''
        self.headers={"sec-ch-ua": '"Google Chrome";v="89", "Chromium";v="89", ";Not A Brand";v="99"',"sec-ch-ua-mobile": "?0","sec-ch-ua-platform": "Linux","sec-fetch-dest": "document","sec-fetch-mode": "navigate","sec-fetch-site": "none","sec-fetch-user": "?1","upgrade-insecure-requests": "1","user-agent":"Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36"}
        self.html = self.get(url)
        self.js = json.loads(re.search('\>(\{\"props\":.*?)\<\/script>',self.html.text).group(1))
        self.account = Account(self.js['props']['pageProps']['itemInfo']['itemStruct']['author'])
        self.video = self.js['props']['pageProps']['itemInfo']['itemStruct']
        self.cover = self.video['video']['cover']
        self.music = self.video['music']['title']
        self.caption = self.video['desc']
        self.create = datetime.fromtimestamp(self.video['createTime'])
        self.url = url
        self.id, self.height, self.width, self.duration, self.ratio,self.bitrate = self.video['video']['id'], self.video['video']['height'],self.video['video']['width'],self.video['video']['duration'],self.video['video']['ratio'],self.video['video']['bitrate']
        self.tt_csrf_token=self.js['query']['$initialProps']['$csrfToken']
        self.aftercsrf=self.js['query']['$initialProps']['$encryptedWebid']
        self.tt_webid_v2=self.js['query']['$initialProps']['$logId']
        self.headers.update({'Cookie':f'tt_webid_v2={self.tt_webid_v2}; tt_csrf_token={self.tt_csrf_token}; {self.aftercsrf}'})
    def __str__(self) -> str:
        return f"<(ID:{self.id})>"
    def __repr__(self) -> str:
        return self.__str__()

class Account:
    def __init__(self, js:dict) -> None:
        self.avatar = js['avatarThumb']
        self.username = js['uniqueId']
        self.nickname = js['nickname']
        self.signature = js['signature']
        self.create = datetime.fromtimestamp(js['createTime'])
        self.verified = js['verified']
        self.private = js["privateAccount"]
    def __repr__(self) -> str:
        return f"<(OWNER:{self.username} VERIFIED:{self.verified})>"
    def __str__(self) -> str:
        return self.__repr__()
class tiktok:
    def __init__(self, url) -> None:
        self.request = Session()
        self.header  = {"content-type": "application/x-www-form-urlencoded; charset=UTF-8","hx-active-element": "submit","hx-current-url": "https://ssstik.io/","hx-request": "true","hx-target": "target","origin": "https://ssstik.io","sec-fetch-dest": "","sec-fetch-mode": "cors","sec-fetch-site": "same-origin","user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36"}
        self.url     = "https://ssstik.io"
        self.html    = self.request.get(self.url, headers=self.header).text
        #self.key     = BeautifulSoup(self.html, "html.parser").find_all("form",attrs={"data-hx-target":"#target"})[0].get("include-vals")
        #self.post    = BeautifulSoup(self.html, "html.parser").find_all("form",attrs={"data-hx-target":"#target"})[0].get("data-hx-post")
        #self.tt      = re.search("tt\:\'(.*?)\'",self.key)[1]
        #self.ts      = re.search("ts\:([0-9]{5,15})",self.key)[1]
        self.url_vid = url
        
    def get_info(self):
        try:
            data   = {"id": self.url_vid,"locale": "en","tt": 0,"ts": 0}
            post   = self.request.post(self.url+re.findall('data-hx-post=\"(.*?)\"',self.html)[0], headers=self.header, data=data)
            respon = BeautifulSoup(post.text, "html.parser")
            #return post
            self.hasil  = {"video":[f'{self.url}{respon.find_all("a",class_="pure-button pure-button-primary is-center u-bl dl-button download_link without_watermark")[0].get("href")}',f'{self.url}{respon.find_all("a",class_="pure-button pure-button-primary is-center u-bl dl-button download_link without_watermark_direct")[0].get("href")}'],"music":f'{respon.find_all("a",class_="pure-button pure-button-primary is-center u-bl dl-button download_link music")[0].get("href")}'}
            return [info_videotiktok(x, self.header, self.request) for x in self.hasil["video"]]
        except IndexError:
            raise InvalidUrl("URL ERROR")
    def __str__(self) -> str:
        return "<[ Method: ssstik.io ]>"
    def __repr__(self) -> str:
        return self.__str__()
class tiktok2:
    def __init__(self, url) -> None:
        self.request = Session()
        self.url = url
        self.header  = {"accept": "*/*","accept-language": "en-US,en;q=0.9,id;q=0.8","origin": "https://snaptik.app","referer": "https://snaptik.app/ID","sec-fetch-dest": None,"sec-fetch-mode": "cors","sec-fetch-site": "same-origin","user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36",}
    def get_info(self):
        try:
            self.request.post("https://snaptik.app/check_token.php", headers=self.header)
            self.php = re.findall(', \"(.*?\.php)',requests.get("https://snaptik.app", headers=self.header).text)[0]
            self.bs = BeautifulSoup(self.request.post(f"https://snaptik.app/{self.php}", headers=self.header, data={"url":self.url}).text, "html.parser")
            self.hasil=[self.request,{"title":self.bs('a', attrs={"title":""})[0].text,"date":self.bs("b", attrs={"class":"blur"})[0].text,"video":list(filter(lambda x:x, map(lambda x:x["href"] if "token" in x["href"] else None, self.bs("a", attrs={"class":"abutton is-success is-fullwidth"}))))}, self.header]
            return [info_videotiktok(x, self.header, self.request) for x in self.hasil[1]["video"]]
        except IndexError:
            raise InvalidUrl("URL ERROR")
    def __str__(self) -> str:
        return "<[ Method: Snaptik.io ]>"
    def __repr__(self) -> str:
        return self.__str__()
class Tiktok:
    info_video = info_post
    ssstik = tiktok
    snaptik = tiktok2
    keeptiktok = keeptiktok
    def __str__(self) -> str:
        return "<[ Tiktok Downloader No Watermark Scraper]>"
    def __repr__(self) -> str:
        return self.__str__()


from io import BytesIO
from typing import Union
from requests import Session
class info_videotiktok:
    def __init__(self,  url, Session:Session, type='video') -> None:
        self.json = url
        self.type = type
        self.Session =Session
    def get_size(self)->int:
        return int(self.Session.get(self.json,  stream=True).headers["Content-Length"])
    def download(self, out=False)->Union[None, BytesIO]:
        if isinstance(out, str):
            open(out, "wb").write(self.Session.get(self.json).content)
        else:
            return BytesIO(self.Session.get(self.json).content)
    def __str__(self) -> str:
        return f"<[type:{self.type}]>"
    def __repr__(self) -> str:
        return self.__str__()


from flask import Flask, request,render_template
from . import info_post, snaptik, ssstik
import json
app = Flask(__name__)
@app.route('/')
def index():
    return render_template('index.html')
@app.route('/test')
def test():
    if request.args.get('s') == 't':
        return json.dumps([{'type':'music','url':''},{'type':'video','url':''}])
    elif request.args.get('s') =='i':
        return json.dumps({
            "account": {
                "avatar": "https://p16-sign-sg.tiktokcdn.com/aweme/100x100/tiktok-obj/0c8589b8821e646047ebd34103ae3aea.jpeg?x-expires=1634824800&x-signature=y8keUWmt6%2FBT8SkZXZfXeiV2bI8%3D",
                "username": "tribunsumselcom",
                "nickname": "Tribun Sumsel",
                "signature": "Official Tiktok of Tribunsumsel.com\nYoutube & FB : Tribun Sumsel",
                "create": 1600221674.0,
                "verified": False
            },
            "music": "suara asli - Tribun Sumsel",
            "caption": "#tiktokberita #viral #infoviral #siswatangerangburonan #buronaninternasional",
            "create": 1634636189.0,
            "url": "https://www.tiktok.com/@tribunsumselcom/video/7020708969563917595",
            "id": "7020708969563917595"
        })
    else:
        return json.dumps({'msg':'test'})
@app.route('/<path:path>')
def snapt(path):
    if path == 'info':
        try:
            if not request.args.get('url'):
                return json.dumps({'msg':'url parameter required'}, indent=4)
            resp=info_post(request.args['url'])
            return json.dumps({
            'account':{
                'avatar':resp.account.avatar,
                'username':resp.account.username,
                'nickname':resp.account.nickname,
                'signature':resp.account.signature,
                'create':resp.account.create.timestamp() if resp.account.create else 0,
                'verified':resp.account.verified
            },
            'music':resp.music,
            'cover':resp.cover,
            'caption':resp.caption,
            'create':resp.create.timestamp(),
            'url':resp.url,
            'id':resp.id    }, indent=4)
        except Exception as e:
            print(e)
            return json.dumps({
                'msg':'url tidak valid'
            })
    elif not path in ['snaptik', 'ssstik']:
        return json.dumps({'msg':'path tidak ditemukan'})
    if request.args.get('url'):
        try:
            ok=snaptik(request.args['url']).get_media() if path == 'snaptik' else ssstik().get_media(request.args['url'])
            return json.dumps([{'type':i.type,'url':i.json} for i in ok],indent=4)
        except Exception as e:
            print(e)
            return json.dumps({
                'msg':'url tidak valid'
            })
    else:
        return json.dumps({'msg':'url parameter required'}, indent=4)


from sys import stderr
from .utils import info_videotiktok
from py_mini_racer import MiniRacer
from .Except import InvalidUrl
from requests import Session
from re import findall
class snaptik(Session):
    '''
    :param tiktok_url:
    ```python
    >>> tik=snaptik('url')
    >>> tik.get_media()
    [<[type:video]>, <[type:video]>]
    ```
    '''
    decoder = MiniRacer()
    decoder.eval('var _0xc38e = [\n    "",\n    "split",\n    "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+/",\n    "slice",\n    "indexOf",\n    "",\n    "",\n    ".",\n    "pow",\n    "reduce",\n    "reverse",\n    "0",\n  ];\n  function _0xe48c(d, e, f) {\n    var g = _0xc38e[2][_0xc38e[1]](_0xc38e[0]);\n    var h = g[_0xc38e[3]](0, e);\n    var i = g[_0xc38e[3]](0, f);\n    var j = d[_0xc38e[1]](_0xc38e[0])\n      [_0xc38e[10]]()\n      [_0xc38e[9]](function (a, b, c) {\n        if (h[_0xc38e[4]](b) !== -1)\n          return (a += h[_0xc38e[4]](b) * Math[_0xc38e[8]](e, c));\n      }, 0);\n    var k = _0xc38e[0];\n    while (j > 0) {\n      k = i[j % f] + k;\n      j = (j - (j % f)) / f;\n    }\n    return k || _0xc38e[11];\n  }\n  \n  function decoder(h, u, n, t, e, r) {\n    r = "";\n    for (var i = 0, len = h.length; i < len; i++) {\n      var s = "";\n      while (h[i] !== n[e]) {\n        s += h[i];\n        i++;\n      }\n      for (var j = 0; j < n.length; j++) s = s.replace(new RegExp(n[j], "g"), j);\n      r += String.fromCharCode(_0xe48c(s, e, 10) - t);\n    }\n    return decodeURIComponent(escape(r));\n  }')
    def __init__(self, tiktok_url:str) -> None:
        super().__init__()
        self.resp = self.get('https://snaptik.app/abc.php', params={'url':tiktok_url,'lang':'en'}, headers={'User-Agent':'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36'})
        if 'error_api_web;' in self.resp.text or 'Error:' in self.resp.text:
            raise InvalidUrl()
    def get_media(self)->list[info_videotiktok]:
        '''
        ```python
        >>> <snaptik object>.get_media()
        [<[type:video]>, <[type:video]>]
        ```
        '''
        stderr.flush()
        d=eval(findall('\(\".*?,.*?,.*?,.*?,.*?.*?\)',self.resp.text)[0]).__str__()
        dec = self.decoder.eval(f"decoder{d}")
        stderr.flush()
        return [info_videotiktok(i, self) for i in set(map(lambda x:x[0].strip('\\'),findall('\"(https?://(tikcdn\.net|snapsave\.info).*?)\"',dec)))]



from distutils.core import setup
from os import path
base_dir = path.abspath(path.dirname(__file__))
setup(
  name = 'tiktok_downloader',        
  packages = ['tiktok_downloader'],
  include_package_data=True, 
  version = '0.1.3',    
  license='MIT',     
  description = 'Tiktok Downloader&Scraper using bs4&requests', 
  author = 'Krypton Byte',                  
  author_email = 'galaxyvplus6434@gmail.com',     
  url = 'https://github.com/krypton-byte/tiktok_downloader',   
  download_url = 'https://github.com/krypton-byte/tiktok_downloader/archive/0.0.7.tar.gz',    
  keywords = ['tiktok', 'downloader', 'scrapper', 'tikdok-scraper', 'tiktok-downloader'], 
  install_requires=[           
          'bs4',
          'flask',
          'cloudscraper',
          'requests',
          'py-mini-racer'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',  
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
  ],
)


import argparse,json, os
import json
from .ssstik import ssstik
from .snaptik import snaptik
from .scrapper import info_post
from sys import stderr
arg=argparse.ArgumentParser(prog=f"python3 -m {os.path.dirname(__file__).split('/')[-1]}", description='Wellcome Maker')
arg.add_argument('--snaptik', action='store_true')
arg.add_argument('--ssstik', action='store_true')
arg.add_argument('--info', action='store_true')
arg.add_argument('--url')
arg.add_argument('--server', action='store_true')
arg.add_argument('--host', type=str, default='0.0.0.0')
arg.add_argument('--debug', action='store_true')
arg.add_argument('--port', default=8000, type=int)
arg.add_argument('--json', action='store_true')
arg.add_argument('--save')
parse=arg.parse_args()
if parse.server:
    from .server import app
    app.run(host=parse.host, port=parse.port, debug=bool(parse.debug))
elif parse.url:
    if parse.snaptik or parse.ssstik:
        try:
            ok=snaptik(parse.url).get_media() if parse.snaptik else ssstik().get_media(parse.url)
            if parse.json or not(parse.save):
                print(json.dumps( [{'type':i.type,'url':i.json} for i in ok],indent=4))
            elif parse.save:
                ok[0].download(parse.save)
            else:
                os.system(f"python3 -m {os.path.dirname(__file__).split('/')[-1]} --help")
        except Exception:
            stderr.write('Post Not Found\n')
            stderr.flush()
    elif parse.info:
        try:
            resp=info_post(parse.url)
            print(json.dumps({
                'account':{
                    'avatar':resp.account.avatar,
                    'username':resp.account.username,
                    'nickname':resp.account.nickname,
                    'signature':resp.account.signature,
                    'create':resp.account.create.timestamp() if resp.account.create else 0,
                    'verified':resp.account.verified
                },
                'music':resp.music,
                'caption':resp.caption,
                'create':resp.create.timestamp(),
                'url':resp.url,
                'id':resp.id    }, indent=4))
        except Exception as e:
            print(e)
            stderr.write('Post Not Found\n')
            stderr.flush()
    else:
        os.system(f"python3 -m {os.path.dirname(__file__).split('/')[-1]} --help")
else:
    os.system(f"python3 -m {os.path.dirname(__file__).split('/')[-1]} --help")